import type { IAutorunOptions, IReactionDisposer, IReactionOptions, IReactionPublic } from 'mobx';
import type { OmitValueOfKey, OmitValueWithType } from 'tslang';
import type { IHistory } from '../history';
import type { Router } from '../router';
import type { NextRouteMatch } from './next-route-match';
import type { GeneralParamDict, RouteMatchSharedOptions } from './route-match-shared';
import { RouteMatchShared } from './route-match-shared';
export declare const ROUTE_SERVICE_ENTER_UPDATE_DATA_SYMBOL: unique symbol;
export declare const ROUTE_SERVICE_UPDATE_DATA_SYMBOL: unique symbol;
export declare const ROUTE_SERVICE_EXTENSION_SYMBOL: unique symbol;
export interface RouteUpdateContext {
    descendants: boolean;
}
/**
 * Route before enter callback.
 * @return Return `true` or `undefined` to do nothing; return `false` to revert
 * this history change; return full path to redirect.
 */
export type RouteBeforeEnterCallback<TRouteMatch extends RouteMatch = RouteMatch> = (next: TRouteMatch['$next']) => Promise<boolean | void> | boolean | void;
/**
 * Route before update callback.
 * @return Return `true` or `undefined` to do nothing; return `false` to revert
 * this history change; return full path to redirect.
 */
export type RouteBeforeUpdateCallback<TRouteMatch extends RouteMatch = RouteMatch> = (next: TRouteMatch['$next'], context: RouteUpdateContext) => Promise<boolean | void> | boolean | void;
export interface RouteBeforeUpdateOptions {
    traceDescendants: boolean;
}
export interface RouteBeforeUpdateEntry<TRouteMatch extends RouteMatch = RouteMatch> {
    callback: RouteBeforeUpdateCallback<TRouteMatch>;
    options?: RouteBeforeUpdateOptions;
}
/**
 * Route before leave callback.
 * @return Return `true` or `undefined` to do nothing; return `false` to revert
 * this history change.
 */
export type RouteBeforeLeaveCallback = () => Promise<boolean | void> | boolean | void;
export type RouteWillEnterCallback<TRouteMatch extends RouteMatch = RouteMatch> = (next: TRouteMatch['$next']) => Promise<void> | void;
export type ServiceWillEnterCallbackReturn<TRouteMatch extends RouteMatch, TClassExtension extends Partial<RouteServiceExtension<TRouteMatch>> = {}> = Omit<RouteServiceExtension<TRouteMatch>, keyof TClassExtension> | void extends infer T ? Promise<T> | T : never;
export type RouteWillUpdateCallback<TRouteMatch extends RouteMatch = RouteMatch> = (next: TRouteMatch['$next'], context: RouteUpdateContext) => Promise<void> | void;
export type ServiceWillUpdateCallbackReturn<TRouteMatch extends RouteMatch, TClassExtension extends Partial<RouteServiceExtension<TRouteMatch>> = {}> = Omit<RouteServiceExtension<TRouteMatch>, keyof TClassExtension> | void extends infer T ? Promise<T> | T : never;
export interface RouteWillUpdateOptions {
    traceDescendants: boolean;
}
export interface RouteWillUpdateEntry<TRouteMatch extends RouteMatch = RouteMatch> {
    callback: RouteWillUpdateCallback<TRouteMatch>;
    options?: RouteWillUpdateOptions;
}
export type RouteWillLeaveCallback = () => Promise<void> | void;
export type RouteEnterCallback = () => void;
export type ServiceEnterCallbackReturn<TRouteMatch extends RouteMatch, TClassExtension extends Partial<RouteServiceExtension<TRouteMatch>> = {}> = Omit<RouteServiceExtension<TRouteMatch>, keyof TClassExtension> | void;
export type RouteUpdateCallback = (context: RouteUpdateContext) => void;
export type ServiceUpdateCallbackReturn<TRouteMatch extends RouteMatch, TClassExtension extends Partial<RouteServiceExtension<TRouteMatch>> = {}> = Omit<RouteServiceExtension<TRouteMatch>, keyof TClassExtension> | void;
export interface RouteUpdateOptions {
    traceDescendants: boolean;
}
export interface RouteUpdateEntry {
    callback: RouteUpdateCallback;
    options?: RouteUpdateOptions;
}
export type RouteLeaveCallback = () => void;
export type RouteAfterEnterCallback = () => void;
/**
 * Route after update callback.
 */
export type RouteAfterUpdateCallback = (context: RouteUpdateContext) => void;
export interface RouteAfterUpdateOptions {
    traceDescendants: boolean;
}
export interface RouteAfterUpdateEntry {
    options?: RouteAfterUpdateOptions;
    callback: RouteAfterUpdateCallback;
}
export type RouteAfterLeaveCallback = () => void;
export type RouteReactiveDisposer = IReactionDisposer;
export type RouteAutorunView = (reaction: IReactionPublic) => void;
export type RouteAutorunOptions = IAutorunOptions;
export type RouteReactionExpression<T> = (reaction: IReactionPublic) => T;
export type RouteReactionEffect<T> = (value: T, previousValue: T, reaction: IReactionPublic) => void;
export type RouteReactionOptions<T, FireImmediately extends boolean = boolean> = IReactionOptions<T, FireImmediately> | undefined;
export type RouteHookRemovalCallback = () => void;
export type RouteBeforeEnterOrUpdateCallback<TRouteMatch extends RouteMatch = RouteMatch> = (next: TRouteMatch['$next']) => Promise<boolean | void> | boolean | void;
export type RouteWillEnterOrUpdateCallback<TRouteMatch extends RouteMatch = RouteMatch> = (next: TRouteMatch['$next']) => Promise<void> | void;
export type RouteAfterEnterOrUpdateCallback = () => void;
export type RouteServiceFactory<TRouteMatch extends RouteMatch, TEnterUpdateData extends object | void, TClassExtension extends Partial<RouteServiceExtension<TRouteMatch>>> = (match: TRouteMatch) => IRouteService<TRouteMatch, TEnterUpdateData, TClassExtension> extends infer T ? Promise<T> | T : never;
export type IRouteService<TRouteMatch extends RouteMatch = RouteMatch, TEnterUpdateData extends object | void = object | void, TClassExtension extends Partial<RouteServiceExtension<TRouteMatch>> = {}> = {
    beforeEnter?(next: TRouteMatch['$next']): Promise<boolean | TEnterUpdateData | void> | boolean | TEnterUpdateData | void;
    willEnter?(next: TRouteMatch['$next'], data: TEnterUpdateData): ServiceWillEnterCallbackReturn<TRouteMatch, TClassExtension>;
    enter?(data: TEnterUpdateData): ServiceEnterCallbackReturn<TRouteMatch, TClassExtension>;
    afterEnter?: RouteAfterEnterCallback;
    beforeUpdate?(next: TRouteMatch['$next'], context: RouteUpdateContext): Promise<boolean | TEnterUpdateData | void> | boolean | TEnterUpdateData | void;
    willUpdate?(next: TRouteMatch['$next'], context: RouteUpdateContext, data: TEnterUpdateData): ServiceWillUpdateCallbackReturn<TRouteMatch, TClassExtension>;
    update?(context: RouteUpdateContext, data: TEnterUpdateData): ServiceUpdateCallbackReturn<TRouteMatch, TClassExtension>;
    afterUpdate?: RouteAfterUpdateCallback;
    beforeLeave?: RouteBeforeLeaveCallback;
    willLeave?: RouteWillLeaveCallback;
    leave?: RouteLeaveCallback;
    afterLeave?: RouteAfterLeaveCallback;
} & TClassExtension;
export type RouteServiceExtension<TRouteMatch extends RouteMatch> = OmitValueWithType<OmitValueOfKey<TRouteMatch, keyof RouteMatch>, RouteMatch, false>;
export interface RouteMatchResult {
    matched: boolean;
    exactlyMatched: boolean;
    rest: string;
}
export interface RouteMatchParallelOptions<TGroupName extends string> {
    groups?: TGroupName[];
    matches?: RouteMatch[];
}
export interface RouteMatchEntry {
    match: RouteMatch;
    exact: boolean;
    segment: string;
    rest: string;
}
export interface RouteSourceQuery {
    id: string | symbol | true;
    value: string;
}
export interface RouteSource {
    groupToMatchToMatchEntryMapMap: Map<string | undefined, Map<RouteMatch, RouteMatchEntry>>;
    queryMap: Map<string, RouteSourceQuery>;
    pathMap: Map<string | undefined, string>;
}
export interface RouteMatchOptions extends RouteMatchSharedOptions {
    exact: boolean | string;
}
export declare class RouteMatch<TParamDict extends GeneralParamDict = GeneralParamDict, TNextRouteMatch extends NextRouteMatch<TParamDict> = NextRouteMatch<TParamDict>, TSpecificGroupName extends string | undefined = string | undefined, TGroupName extends string = string, TMetadata extends object = object> extends RouteMatchShared<TParamDict, TSpecificGroupName, TGroupName, TMetadata> {
    readonly $parent: RouteMatch | undefined;
    readonly $next: TNextRouteMatch;
    constructor(name: string, router: Router<TGroupName>, source: RouteSource, parent: RouteMatch | undefined, extension: object | undefined, history: IHistory, { exact, ...sharedOptions }: RouteMatchOptions);
    $beforeEnter(callback: RouteBeforeEnterCallback<this>): RouteHookRemovalCallback;
    $beforeUpdate(callback: RouteBeforeUpdateCallback<this>, options?: RouteBeforeUpdateOptions): RouteHookRemovalCallback;
    $beforeLeave(callback: RouteBeforeLeaveCallback): RouteHookRemovalCallback;
    $willEnter(callback: RouteWillEnterCallback): RouteHookRemovalCallback;
    $willUpdate(callback: RouteWillUpdateCallback, options?: RouteWillUpdateOptions): RouteHookRemovalCallback;
    $willLeave(callback: RouteWillLeaveCallback): RouteHookRemovalCallback;
    $enter(callback: RouteEnterCallback): RouteHookRemovalCallback;
    $update(callback: RouteUpdateCallback, options?: RouteUpdateOptions): RouteHookRemovalCallback;
    $leave(callback: RouteLeaveCallback): RouteHookRemovalCallback;
    $afterEnter(callback: RouteAfterEnterCallback): RouteHookRemovalCallback;
    $afterUpdate(callback: RouteAfterUpdateCallback, options?: RouteAfterUpdateOptions): RouteHookRemovalCallback;
    $afterLeave(callback: RouteAfterLeaveCallback): RouteHookRemovalCallback;
    $autorun(view: RouteAutorunView, options?: RouteAutorunOptions): RouteHookRemovalCallback;
    $reaction<T>(expression: RouteReactionExpression<T>, effect: RouteReactionEffect<T>, options?: RouteReactionOptions<T>): RouteHookRemovalCallback;
    $beforeEnterOrUpdate(callback: RouteBeforeEnterOrUpdateCallback<this>, beforeUpdateOptions?: RouteBeforeUpdateOptions): RouteHookRemovalCallback;
    $willEnterOrUpdate(callback: RouteWillEnterOrUpdateCallback<this>, willUpdateOptions?: RouteWillUpdateOptions): RouteHookRemovalCallback;
    $afterEnterOrUpdate(callback: RouteAfterEnterOrUpdateCallback, afterUpdateOptions?: RouteAfterUpdateOptions): RouteHookRemovalCallback;
    $service<TEnterUpdateData extends object | void, TClassExtension extends Partial<RouteServiceExtension<this>>>(factory: RouteServiceFactory<this, TEnterUpdateData, TClassExtension>): this;
    $parallel(options: RouteMatchParallelOptions<TGroupName>): void;
    $match(path: string): RouteMatchResult;
    static SEGMENT: RegExp;
    static REST: RegExp;
}
