"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Router = void 0;
const tslib_1 = require("tslib");
const hyphenate_1 = tslib_1.__importDefault(require("hyphenate"));
const lodash_1 = tslib_1.__importDefault(require("lodash"));
const mobx_1 = require("mobx");
const _utils_1 = require("./@utils");
const history_1 = require("./history");
const route_builder_1 = require("./route-builder");
const route_match_1 = require("./route-match");
const DEFAULT_SEGMENT_MATCHER_CALLBACK = key => (0, hyphenate_1.default)(key, { lowerCase: true });
class Router {
    constructor(history, { readOnly = false, listen = readOnly ? false : true, segmentMatcher, } = {}) {
        /** @internal */
        Object.defineProperty(this, "_history", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "$readOnly", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        /** @internal */
        Object.defineProperty(this, "_groupToRouteMatchMap", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: new Map()
        });
        /** @internal */
        Object.defineProperty(this, "_segmentMatcher", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        /** @internal */
        Object.defineProperty(this, "_snapshot", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        /** @internal */
        Object.defineProperty(this, "_nextSnapshot", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        /** @internal */
        Object.defineProperty(this, "_source", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: (0, mobx_1.observable)({
                groupToMatchToMatchEntryMapMap: new Map(),
                queryMap: new Map(),
                pathMap: new Map(),
            })
        });
        /** @internal */
        Object.defineProperty(this, "_matchingSource", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: (0, mobx_1.observable)({
                groupToMatchToMatchEntryMapMap: new Map(),
                queryMap: new Map(),
                pathMap: new Map(),
            })
        });
        /** @internal */
        Object.defineProperty(this, "_changing", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: Promise.resolve()
        });
        /** @internal */
        Object.defineProperty(this, "_routing", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: 0
        });
        /** @internal */
        Object.defineProperty(this, "_onHistoryChange", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: (snapshot) => {
                (0, mobx_1.runInAction)(() => {
                    this._nextSnapshot = snapshot;
                    this._routing++;
                });
                this._changing = this._changing
                    .then(() => this._asyncOnHistoryChange(snapshot))
                    .finally(() => {
                    (0, mobx_1.runInAction)(() => {
                        this._routing--;
                    });
                })
                    .catch(console.error);
            }
        });
        /** @internal */
        Object.defineProperty(this, "_asyncOnHistoryChange", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: async (nextSnapshot) => {
                const snapshot = this._snapshot;
                if (snapshot && lodash_1.default.isEqual(snapshot, nextSnapshot)) {
                    return;
                }
                if (this._isNextSnapshotOutDated(nextSnapshot)) {
                    return;
                }
                const { ref, data } = (0, history_1.getActiveHistoryEntry)(nextSnapshot);
                const navigateCompleteListener = data && data.navigateCompleteListener;
                this._updateMatchingSource(ref);
                const groupToMatchToMatchEntryMapMap = this._matchingSource.groupToMatchToMatchEntryMapMap;
                const interUpdateDataArray = await Promise.all(this._generalGroups.map(async (group) => this._beforeUpdate(nextSnapshot, group, groupToMatchToMatchEntryMapMap.get(group))));
                if (interUpdateDataArray.some(data => !data)) {
                    return;
                }
                await Promise.all(interUpdateDataArray.map(data => this._willUpdate(data)));
                (0, mobx_1.runInAction)(() => {
                    this._snapshot = nextSnapshot;
                    this._update(interUpdateDataArray);
                });
                await Promise.all(interUpdateDataArray.map(data => this._afterUpdate(data)));
                if (navigateCompleteListener) {
                    navigateCompleteListener();
                }
            }
        });
        (0, mobx_1.makeObservable)(this);
        this._history = history;
        this.$readOnly = readOnly;
        this._segmentMatcher = segmentMatcher || DEFAULT_SEGMENT_MATCHER_CALLBACK;
        if (listen) {
            this.$listen();
        }
    }
    get $routing() {
        return this._routing > 0;
    }
    get $snapshot() {
        return this._snapshot;
    }
    get $nextSnapshot() {
        return this._nextSnapshot;
    }
    get $current() {
        return new route_builder_1.RouteBuilder(this, 'current');
    }
    get $next() {
        return new route_builder_1.RouteBuilder(this, 'next');
    }
    get $groups() {
        return Array.from(this._groupToRouteMatchMap.keys()).filter((group) => !!group);
    }
    /** @internal */
    get _generalGroups() {
        return [undefined, ...this.$groups];
    }
    $route(groupOrSchema, schemaOrUndefined) {
        let group;
        let schema;
        if (typeof groupOrSchema === 'string') {
            group = groupOrSchema;
            schema = schemaOrUndefined;
        }
        else {
            group = undefined;
            schema = groupOrSchema;
        }
        const [routeMatch] = this._buildRouteMatch(group, '', undefined, undefined, {
            $exact: true,
            $match: '',
            ...schema,
        });
        this._groupToRouteMatchMap.set(group, routeMatch);
        return routeMatch;
    }
    $listen() {
        const history = this._history;
        if (this.$readOnly) {
            this._snapshot = history.snapshot;
            this._updateMatchingSource(history.ref);
            this._update([]);
            return () => { };
        }
        else {
            return history.listen(this._onHistoryChange);
        }
    }
    $ref() {
        return this.$current.$ref();
    }
    $href() {
        return this.$current.$href();
    }
    $(route, params) {
        const buildingPart = typeof route === 'string'
            ? route
            : {
                route,
                params,
            };
        return new route_builder_1.RouteBuilder(this, 'current', [buildingPart]);
    }
    $scratch() {
        return new route_builder_1.RouteBuilder(this, 'none');
    }
    $push(ref, options) {
        this.$current.$(ref).$push(options);
    }
    $replace(ref, options) {
        this.$current.$(ref).$replace(options);
    }
    /** @internal */
    _push(ref, { onComplete } = {}) {
        this._assertNonReadOnly();
        this._history
            .push(ref, { navigateCompleteListener: onComplete })
            .catch(console.error);
    }
    /** @internal */
    _replace(ref, { onComplete } = {}) {
        this._assertNonReadOnly();
        this._history
            .replace(ref, { navigateCompleteListener: onComplete })
            .catch(console.error);
    }
    /** @internal */
    _updateMatchingSource(ref) {
        const { pathname, search } = (0, _utils_1.parseRef)(ref);
        const queryMap = (0, _utils_1.parseSearch)(search);
        const pathMap = new Map();
        pathMap.set(undefined, pathname || '/');
        const groups = this.$groups;
        // Extract group route paths in query
        for (const group of groups) {
            const key = `_${group}`;
            if (!queryMap.has(key)) {
                continue;
            }
            const path = queryMap.get(key);
            if (path) {
                pathMap.set(group, path);
            }
            queryMap.delete(key);
        }
        // Match parallel routes
        const groupToMatchEntriesMap = new Map();
        const groupToRouteMatchMap = this._groupToRouteMatchMap;
        for (const [group, path] of pathMap) {
            const routeMatch = groupToRouteMatchMap.get(group);
            const routeMatchEntries = this._match([routeMatch], path) || [];
            if (!routeMatchEntries.length) {
                continue;
            }
            const [{ match }] = routeMatchEntries;
            if (match.$group !== group) {
                continue;
            }
            groupToMatchEntriesMap.set(group, routeMatchEntries);
        }
        // Check primary match parallel options
        const groupToMatchToMatchEntryMapMap = new Map();
        const primaryMatchEntries = groupToMatchEntriesMap.get(undefined);
        {
            const primaryMatch = primaryMatchEntries === null || primaryMatchEntries === void 0 ? void 0 : primaryMatchEntries[primaryMatchEntries.length - 1].match;
            const options = primaryMatch === null || primaryMatch === void 0 ? void 0 : primaryMatch._parallel;
            const { groups = [], matches = [] } = options || {};
            for (const [group, entries] of groupToMatchEntriesMap) {
                if (!group ||
                    !options ||
                    groups.includes(group) ||
                    entries.some(({ match }) => matches.includes(match))) {
                    groupToMatchToMatchEntryMapMap.set(group, new Map(entries.map((entry) => [
                        entry.match,
                        entry,
                    ])));
                }
            }
        }
        const matchingSource = this._matchingSource;
        (0, mobx_1.runInAction)(() => {
            matchingSource.groupToMatchToMatchEntryMapMap =
                groupToMatchToMatchEntryMapMap;
            matchingSource.pathMap = pathMap;
            const matchingQueryKeyToIdMap = new Map(lodash_1.default.flatMap(Array.from(groupToRouteMatchMap.values()).reverse(), route => [...route.$next.$rest._queryKeyToIdMap]));
            matchingSource.queryMap = new Map(lodash_1.default.compact(Array.from(queryMap).map(([key, value]) => matchingQueryKeyToIdMap.has(key)
                ? [key, { id: matchingQueryKeyToIdMap.get(key), value }]
                : undefined)));
        });
    }
    /** @internal */
    async _beforeUpdate(nextSnapshot, group, matchToMatchEntryMap) {
        if (!matchToMatchEntryMap) {
            matchToMatchEntryMap = new Map();
        }
        // Prepare previous/next match set
        let previousMatchToMatchEntryMap = this._source.groupToMatchToMatchEntryMapMap.get(group);
        if (!previousMatchToMatchEntryMap) {
            previousMatchToMatchEntryMap = new Map();
            (0, mobx_1.runInAction)(() => {
                this._source.groupToMatchToMatchEntryMapMap.set(group, previousMatchToMatchEntryMap);
            });
        }
        const previousMatchSet = new Set(previousMatchToMatchEntryMap.keys());
        const matchSet = new Set(matchToMatchEntryMap.keys());
        const leavingMatchSet = new Set(previousMatchSet);
        for (const match of matchSet) {
            leavingMatchSet.delete(match);
        }
        const reversedLeavingMatches = Array.from(leavingMatchSet).reverse();
        const enteringAndUpdatingMatchSet = new Set(matchSet);
        const descendantUpdatingMatchSet = new Set();
        for (const match of previousMatchSet) {
            if (!enteringAndUpdatingMatchSet.has(match)) {
                continue;
            }
            const nextMatch = match.$next;
            if (lodash_1.default.isEqual(match._pathSegments, nextMatch._pathSegments) &&
                match.$exact === nextMatch.$exact) {
                if (match._rest === nextMatch._rest) {
                    enteringAndUpdatingMatchSet.delete(match);
                }
                else {
                    descendantUpdatingMatchSet.add(match);
                }
            }
        }
        for (const match of reversedLeavingMatches) {
            const result = await match._beforeLeave();
            if (this._isNextSnapshotOutDated(nextSnapshot)) {
                return undefined;
            }
            if (!result) {
                this._revert();
                return undefined;
            }
        }
        for (const match of enteringAndUpdatingMatchSet) {
            const update = previousMatchSet.has(match);
            const result = update
                ? await match._beforeUpdate(descendantUpdatingMatchSet.has(match))
                : await match._beforeEnter();
            if (!result) {
                match._abortEnterUpdate();
            }
            if (this._isNextSnapshotOutDated(nextSnapshot)) {
                return undefined;
            }
            if (!result) {
                this._revert();
                return undefined;
            }
        }
        return {
            reversedLeavingMatches,
            enteringAndUpdatingMatchSet,
            previousMatchSet,
            descendantUpdatingMatchSet,
        };
    }
    /** @internal */
    async _willUpdate({ reversedLeavingMatches, enteringAndUpdatingMatchSet, previousMatchSet, descendantUpdatingMatchSet, }) {
        for (const match of reversedLeavingMatches) {
            await match._willLeave();
        }
        for (const match of enteringAndUpdatingMatchSet) {
            const update = previousMatchSet.has(match);
            if (update) {
                await match._willUpdate(descendantUpdatingMatchSet.has(match));
            }
            else {
                await match._willEnter();
            }
        }
    }
    /** @internal */
    _update(dataArray) {
        const source = this._source;
        const matchingSource = this._matchingSource;
        source.queryMap = matchingSource.queryMap;
        for (const group of this._generalGroups) {
            const path = matchingSource.pathMap.get(group);
            if (path) {
                source.pathMap.set(group, path);
            }
            else {
                source.pathMap.delete(group);
            }
            const matchToMatchEntryMap = matchingSource.groupToMatchToMatchEntryMapMap.get(group);
            source.groupToMatchToMatchEntryMapMap.set(group, matchToMatchEntryMap);
        }
        for (const { reversedLeavingMatches, enteringAndUpdatingMatchSet, previousMatchSet, descendantUpdatingMatchSet, } of dataArray) {
            for (const match of reversedLeavingMatches) {
                match._leave();
            }
            for (const match of enteringAndUpdatingMatchSet) {
                const update = previousMatchSet.has(match);
                if (update) {
                    match._update(descendantUpdatingMatchSet.has(match));
                }
                else {
                    match._enter();
                }
            }
        }
    }
    /** @internal */
    async _afterUpdate({ reversedLeavingMatches, enteringAndUpdatingMatchSet, previousMatchSet, descendantUpdatingMatchSet, }) {
        for (const match of reversedLeavingMatches) {
            await match._afterLeave();
        }
        for (const match of enteringAndUpdatingMatchSet) {
            const update = previousMatchSet.has(match);
            if (update) {
                await match._afterUpdate(descendantUpdatingMatchSet.has(match));
            }
            else {
                await match._afterEnter();
            }
        }
    }
    /** @internal */
    _isNextSnapshotOutDated(snapshot) {
        return this._nextSnapshot !== snapshot;
    }
    /** @internal */
    _revert() {
        const snapshot = this._snapshot;
        if (snapshot) {
            this._history.restore(snapshot).catch(console.error);
        }
        else {
            this._history.replace('/').catch(console.error);
        }
    }
    /** @internal */
    _match(routeMatches, upperRest) {
        for (const routeMatch of routeMatches) {
            const { matched, exactlyMatched, segment, rest } = routeMatch._match(upperRest);
            if (!matched) {
                continue;
            }
            if (rest === '') {
                return [
                    {
                        match: routeMatch,
                        segment: segment,
                        exact: exactlyMatched,
                        rest,
                    },
                ];
            }
            const result = this._match(routeMatch._children || [], rest);
            if (!result) {
                continue;
            }
            return [
                {
                    match: routeMatch,
                    segment: segment,
                    exact: exactlyMatched,
                    rest,
                },
                ...result,
            ];
        }
        return undefined;
    }
    /** @internal */
    _buildRouteMatches(group, schemaDict, parent, matchingParent) {
        return Object.entries(schemaDict).reduce(([routeMatches, nextRouteMatches], [routeName, schema]) => {
            if (typeof schema === 'boolean') {
                schema = {};
            }
            const [routeMatch, nextRouteMatch] = this._buildRouteMatch(group, routeName, parent, matchingParent, schema);
            parent[routeName] = routeMatch;
            matchingParent[routeName] = nextRouteMatch;
            return [
                [...routeMatches, routeMatch],
                [...nextRouteMatches, nextRouteMatch],
            ];
        }, [[], []]);
    }
    /** @internal */
    _buildRouteMatch(group, routeName, parent, matchingParent, { $match: match = this._segmentMatcher(routeName), $exact: exact = false, $query: queryDict, $children: children, $extension: extension, $metadata: metadata, }) {
        const source = this._source;
        const matchingSource = this._matchingSource;
        const history = this._history;
        const query = new Map(Object.entries(queryDict !== null && queryDict !== void 0 ? queryDict : {}));
        const options = {
            match,
            query,
            exact,
            group,
            metadata,
        };
        const routeMatch = new route_match_1.RouteMatch(routeName, this, source, parent, extension, history, options);
        const nextRouteMatch = new route_match_1.NextRouteMatch(routeName, this, matchingSource, matchingParent, routeMatch, history, options);
        routeMatch.$next = nextRouteMatch;
        if (children) {
            const [childRouteMatches, childNextRouteMatches] = this._buildRouteMatches(group, children, routeMatch, nextRouteMatch);
            routeMatch._children = childRouteMatches;
            nextRouteMatch._children = childNextRouteMatches;
        }
        return [routeMatch, nextRouteMatch];
    }
    /** @internal */
    _assertNonReadOnly() {
        if (this.$readOnly) {
            throw new Error('Router is read-only');
        }
    }
}
tslib_1.__decorate([
    mobx_1.observable.ref
], Router.prototype, "_snapshot", void 0);
tslib_1.__decorate([
    mobx_1.observable.ref
], Router.prototype, "_nextSnapshot", void 0);
tslib_1.__decorate([
    mobx_1.observable
], Router.prototype, "_routing", void 0);
exports.Router = Router;
//# sourceMappingURL=router.js.map