export function isQueryIdsMatched(a, b) {
    return a === true || b === true || a === b;
}
export function buildPath(segmentDict, paramDict = {}) {
    return (Object.entries(segmentDict)
        .map(([key, defaultSegment]) => {
        const param = paramDict[key];
        const segment = typeof param === 'string' ? param : defaultSegment;
        if (typeof segment !== 'string') {
            throw new Error(`Parameter "${key}" is required`);
        }
        return `/${segment}`;
    })
        .join('') || '/');
}
export function buildRef(pathMap, queryMap) {
    var _a;
    const primaryPath = (_a = pathMap.get(undefined)) !== null && _a !== void 0 ? _a : '';
    const pathQuery = encodeURI(Array.from(pathMap)
        .filter(([group]) => group !== undefined)
        .map(([group, path]) => `_${group}=${path}`)
        .join('&'));
    const normalQuery = queryMap &&
        new URLSearchParams(Array.from(queryMap).filter((entry) => entry[1] !== undefined)).toString();
    const query = pathQuery
        ? normalQuery
            ? `${pathQuery}&${normalQuery}`
            : pathQuery
        : normalQuery;
    return `${primaryPath}${query ? `?${query}` : ''}`;
}
export function parseRef(ref) {
    let pathname = ref || '/';
    let search = '';
    let hash = '';
    const hashIndex = pathname.indexOf('#');
    if (hashIndex >= 0) {
        hash = pathname.slice(hashIndex);
        pathname = pathname.slice(0, hashIndex);
    }
    const searchIndex = pathname.indexOf('?');
    if (searchIndex >= 0) {
        search = pathname.slice(searchIndex);
        pathname = pathname.slice(0, searchIndex);
    }
    return {
        pathname,
        search: search === '?' ? '' : search,
        hash: hash === '#' ? '' : hash,
    };
}
export function parseSearch(search) {
    const searchParams = new URLSearchParams(search);
    return new Map(searchParams);
}
export function testPathPrefix(path, prefix) {
    return (path.startsWith(prefix) &&
        (path.length === prefix.length || path[prefix.length] === '/'));
}
export function tolerate(fn, ...args) {
    let ret;
    try {
        ret = fn(...args);
    }
    catch (error) {
        console.error(error);
        return undefined;
    }
    if (!(ret instanceof Promise)) {
        return ret;
    }
    return ret.catch(error => {
        console.error(error);
    });
}
//# sourceMappingURL=@utils.js.map