import { AbstractHistory, getActiveHistoryEntryIndex } from './history';
const SNAP_PROMISE = Promise.resolve();
export class MemoryHistory extends AbstractHistory {
    constructor({ initialRef = '/', prefix = '' } = {}) {
        super();
        Object.defineProperty(this, "_snapshot", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "lastId", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "prefix", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        this.prefix = prefix;
        if (!initialRef.startsWith('/')) {
            initialRef = `/${initialRef}`;
        }
        const id = 0;
        this._snapshot = {
            entries: [
                {
                    id,
                    ref: initialRef,
                    data: undefined,
                },
            ],
            active: id,
        };
        this.lastId = id;
    }
    get snapshot() {
        return this._snapshot;
    }
    getHRefByRef(ref) {
        return `${this.prefix}${ref}`;
    }
    getRefByHRef(href) {
        const prefix = this.prefix;
        return href.startsWith(prefix) ? href.slice(prefix.length) : href;
    }
    async back() {
        await SNAP_PROMISE;
        let snapshot = this._snapshot;
        const { entries } = snapshot;
        const index = getActiveHistoryEntryIndex(snapshot);
        if (index <= 0) {
            return;
        }
        snapshot = {
            entries,
            active: entries[index - 1].id,
        };
        this._snapshot = snapshot;
        this.emitChange(snapshot);
    }
    async forward() {
        await SNAP_PROMISE;
        let snapshot = this._snapshot;
        const { entries } = snapshot;
        const index = getActiveHistoryEntryIndex(snapshot);
        if (index >= entries.length - 1) {
            return;
        }
        snapshot = {
            entries,
            active: entries[index + 1].id,
        };
        this._snapshot = snapshot;
        this.emitChange(snapshot);
    }
    async push(ref, data) {
        const id = ++this.lastId;
        let snapshot = this._snapshot;
        const { entries } = snapshot;
        const index = getActiveHistoryEntryIndex(snapshot);
        snapshot = {
            entries: [...entries.slice(0, index + 1), { id, ref, data }],
            active: id,
        };
        this._snapshot = snapshot;
        this.emitChange(snapshot);
    }
    async replace(ref, data) {
        let snapshot = this._snapshot;
        const { entries, active: activeId } = snapshot;
        const index = getActiveHistoryEntryIndex(snapshot);
        snapshot = {
            entries: [
                ...entries.slice(0, index),
                { id: activeId, ref, data },
                ...entries.slice(index + 1),
            ],
            active: activeId,
        };
        this._snapshot = snapshot;
        this.emitChange(snapshot);
    }
    async restore(snapshot, toEmitChange = false) {
        this._snapshot = snapshot;
        if (toEmitChange) {
            this.emitChange(this._snapshot);
        }
    }
}
//# sourceMappingURL=memory-history.js.map