import { __decorate } from "tslib";
import { computed, makeObservable } from 'mobx';
import { isQueryIdsMatched } from '../@utils';
import { RouteBuilder } from '../route-builder';
export class RouteMatchShared {
    constructor(name, router, source, parent, history, { match, query, group, metadata }) {
        var _a;
        /**
         * Name of this `RouteMatch`, correspondent to the field name of route
         * schema.
         */
        Object.defineProperty(this, "$name", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        /**
         * Group of this `RouteMatch`, specified in the root route.
         */
        Object.defineProperty(this, "$group", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "$metadata", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        /**
         * Parent of this route match.
         */
        Object.defineProperty(this, "$parent", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "$router", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        /** @internal */
        Object.defineProperty(this, "_source", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        /** @internal */
        Object.defineProperty(this, "_queryKeyToIdMap", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        /** @internal */
        Object.defineProperty(this, "_children", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        /** @internal */
        Object.defineProperty(this, "_history", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        /** @internal */
        Object.defineProperty(this, "_matchPattern", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        makeObservable(this);
        this.$name = name;
        this.$group = group;
        this.$parent = parent;
        this.$router = router;
        this._source = source;
        this._history = history;
        if (match instanceof RegExp && match.global) {
            throw new Error('Expecting a non-global regular expression as match pattern');
        }
        this._matchPattern = match;
        this._queryKeyToIdMap = new Map([
            ...((_a = parent === null || parent === void 0 ? void 0 : parent._queryKeyToIdMap) !== null && _a !== void 0 ? _a : []),
            ...query,
        ]);
        // eslint-disable-next-line @mufan/no-object-literal-type-assertion
        this.$metadata = {
            ...parent === null || parent === void 0 ? void 0 : parent.$metadata,
            ...metadata,
        };
    }
    /**
     * A dictionary of the combination of query string and segments.
     */
    get $params() {
        // eslint-disable-next-line @mufan/no-object-literal-type-assertion
        return {
            ...this._paramSegments,
            ...this._query,
        };
    }
    /**
     * A reactive value indicates whether this route is exactly matched.
     */
    get $exact() {
        const entry = this._matchEntry;
        return !!entry && entry.exact;
    }
    /**
     * A reactive value indicates whether this route is matched.
     */
    get $matched() {
        return !!this._matchEntry;
    }
    /**
     * Get the deepest matching descendant.
     */
    get $rest() {
        if (!this.$matched) {
            return this;
        }
        const children = this._children;
        const matchingChild = children && children.find(match => match.$matched);
        return matchingChild ? matchingChild.$rest : this;
    }
    /** @internal */
    get _matchEntry() {
        return this._getMatchEntry(this._source);
    }
    /** @internal */
    get _segment() {
        const entry = this._matchEntry;
        return entry && entry.segment;
    }
    /** @internal */
    get _paramSegments() {
        const parent = this.$parent;
        const upperSegmentDict = parent && parent._paramSegments;
        const matchPattern = this._matchPattern;
        const segment = this._segment;
        if (!(matchPattern instanceof RegExp)) {
            return {
                ...upperSegmentDict,
            };
        }
        return {
            ...upperSegmentDict,
            [this.$name]: segment,
        };
    }
    /** @internal */
    get _pathSegments() {
        const parent = this.$parent;
        const upperSegmentDict = parent && parent._pathSegments;
        const name = this.$name;
        const matchPattern = this._matchPattern;
        const segment = this._segment;
        return {
            ...upperSegmentDict,
            ...(name
                ? { [name]: typeof matchPattern === 'string' ? matchPattern : segment }
                : undefined),
        };
    }
    /** @internal */
    get _rest() {
        const entry = this._matchEntry;
        return entry ? entry.rest : '';
    }
    /** @internal */
    get _query() {
        const sourceQueryMap = this._source.queryMap;
        return Array.from(this._queryKeyToIdMap).reduce((dict, [key, id]) => {
            const sourceQuery = sourceQueryMap.get(key);
            if (!sourceQuery || !isQueryIdsMatched(sourceQuery.id, id)) {
                return dict;
            }
            dict[key] = sourceQuery.value;
            return dict;
            // eslint-disable-next-line @mufan/no-object-literal-type-assertion
        }, {});
    }
    $(params) {
        return new RouteBuilder(this.$router, 'none', [
            { route: this, params },
        ]);
    }
    $ref(params) {
        return this.$(params).$ref();
    }
    $href(params) {
        return this.$(params).$href();
    }
    $push(params, { onComplete, ...options } = {}) {
        this._build(params, options).$push({ onComplete });
    }
    $replace(params, { onComplete, ...options } = {}) {
        this._build(params, options).$replace({ onComplete });
    }
    /** @internal */
    _build(params = {}, { leave = false, leaves = [] } = {}) {
        if (typeof leaves === 'string') {
            leaves = [leaves];
        }
        if (leave) {
            const group = this.$group;
            if (group === undefined) {
                throw new Error('Cannot leave primary route');
            }
            leaves.push(group);
        }
        return this._getBuilder()
            .$(this, params)
            .$leave(leaves);
    }
}
__decorate([
    computed
], RouteMatchShared.prototype, "$params", null);
__decorate([
    computed
], RouteMatchShared.prototype, "$rest", null);
__decorate([
    computed
], RouteMatchShared.prototype, "_matchEntry", null);
__decorate([
    computed
], RouteMatchShared.prototype, "_segment", null);
__decorate([
    computed
], RouteMatchShared.prototype, "_paramSegments", null);
__decorate([
    computed
], RouteMatchShared.prototype, "_pathSegments", null);
__decorate([
    computed
], RouteMatchShared.prototype, "_rest", null);
__decorate([
    computed
], RouteMatchShared.prototype, "_query", null);
//# sourceMappingURL=route-match-shared.js.map